@@ -1,38 +1,134 @@
 from __future__ import absolute_import
 
 import functools
-import six
 
-from . import blocks
-from . import legos
-from . import pedrpc
-from . import primitives
-from . import exception
-from .blocks import REQUESTS
-from .blocks.block import Block
-from .blocks.checksum import Checksum
-from .blocks.repeat import Repeat
-from .blocks.request import Request
-from .blocks.size import Size
-from .constants import BIG_ENDIAN, LITTLE_ENDIAN, DEFAULT_PROCMON_PORT
+import six
+from past.builtins import map
+
+from . import blocks, exception, legos, pedrpc, primitives
+from .blocks import Block, Checksum, Repeat, Request, REQUESTS, Size
+from .constants import BIG_ENDIAN, DEFAULT_PROCMON_PORT, LITTLE_ENDIAN
 from .event_hook import EventHook
+from .exception import MustImplementException, SizerNotUtilizedError, SullyRuntimeError
 from .fuzz_logger import FuzzLogger
 from .fuzz_logger_csv import FuzzLoggerCsv
-from .fuzz_logger_text import FuzzLoggerText
 from .fuzz_logger_curses import FuzzLoggerCurses
+from .fuzz_logger_text import FuzzLoggerText
 from .ifuzz_logger import IFuzzLogger
 from .ifuzz_logger_backend import IFuzzLoggerBackend
 from .itarget_connection import ITargetConnection
-from .primitives import (BasePrimitive, Delim, Group,
-                         RandomData, Static, String, BitField,
-                         Byte, Word, DWord, QWord, FromFile, Mirror)
+from .primitives import (
+    BasePrimitive,
+    BitField,
+    Byte,
+    Bytes,
+    Delim,
+    DWord,
+    FromFile,
+    Group,
+    Mirror,
+    QWord,
+    RandomData,
+    Static,
+    String,
+    Word,
+)
 from .serial_connection import SerialConnection
-from .sessions import Session, Target, open_test_run
-from .exception import SullyRuntimeError, SizerNotUtilizedError, MustImplementException
+from .sessions import open_test_run, Session, Target
 from .socket_connection import SocketConnection
-from past.builtins import map
-from builtins import chr
-__version__ = '0.1.5'
+
+__all__ = [
+    "BasePrimitive",
+    "BIG_ENDIAN",
+    "BitField",
+    "Block",
+    "blocks",
+    "Byte",
+    "Bytes",
+    "Checksum",
+    "DEFAULT_PROCMON_PORT",
+    "Delim",
+    "DWord",
+    "EventHook",
+    "exception",
+    "FromFile",
+    "FuzzLogger",
+    "FuzzLoggerCsv",
+    "FuzzLoggerCurses",
+    "FuzzLoggerText",
+    "Group",
+    "IFuzzLogger",
+    "IFuzzLoggerBackend",
+    "ITargetConnection",
+    "legos",
+    "LITTLE_ENDIAN",
+    "Mirror",
+    "MustImplementException",
+    "open_test_run",
+    "pedrpc",
+    "primitives",
+    "QWord",
+    "RandomData",
+    "Repeat",
+    "Request",
+    "REQUESTS",
+    "s_bigword",
+    "s_binary",
+    "s_bit",
+    "s_bit_field",
+    "s_bits",
+    "s_block",
+    "s_block_end",
+    "s_block_start",
+    "s_byte",
+    "s_bytes",
+    "s_char",
+    "s_checksum",
+    "s_cstring",
+    "s_delim",
+    "s_double",
+    "s_dunno",
+    "s_dword",
+    "s_from_file",
+    "s_get",
+    "s_group",
+    "s_hex_dump",
+    "s_initialize",
+    "s_int",
+    "s_intelword",
+    "s_lego",
+    "s_long",
+    "s_mirror",
+    "s_mutate",
+    "s_num_mutations",
+    "s_qword",
+    "s_random",
+    "s_raw",
+    "s_render",
+    "s_repeat",
+    "s_repeater",
+    "s_short",
+    "s_size",
+    "s_sizer",
+    "s_static",
+    "s_string",
+    "s_switch",
+    "s_unknown",
+    "s_update",
+    "s_word",
+    "SerialConnection",
+    "Session",
+    "Size",
+    "SizerNotUtilizedError",
+    "SocketConnection",
+    "Static",
+    "String",
+    "SullyRuntimeError",
+    "Target",
+    "Word",
+]
+
+__version__ = "0.1.6"
 
 
 # REQUEST MANAGEMENT
@@ -180,8 +276,8 @@ def s_block(name, group=None, encoder=None, dep=None, dep_value=None, dep_values
 
 def s_block_start(name, *args, **kwargs):
     """
-    Open a new block under the current request. This routine always returns an instance so you can make your fuzzer pretty
-    with indenting::
+    Open a new block under the current request. This routine always returns an instance so you can make your fuzzer
+    pretty with indenting::
 
         if s_block_start("header"):
             s_static("\\x00\\x01")
@@ -209,21 +305,30 @@ def s_block_end(name=None):
     blocks.CURRENT.pop()
 
 
-def s_checksum(block_name, algorithm="crc32", length=0, endian=LITTLE_ENDIAN, fuzzable=True, name=None,
-               ipv4_src_block_name=None,
-               ipv4_dst_block_name=None):
+def s_checksum(
+    block_name,
+    algorithm="crc32",
+    length=0,
+    endian=LITTLE_ENDIAN,
+    fuzzable=True,
+    name=None,
+    ipv4_src_block_name=None,
+    ipv4_dst_block_name=None,
+):
     """
     Create a checksum block bound to the block with the specified name. You *can not* create a checksum for any
     currently open blocks.
 
     :type  block_name: str
-    :param block_name: Name of block to apply sizer to
+    :param block_name: Name of block for checksum calculations
 
-    :type  algorithm:  str
-    :param algorithm:  (Optional, def=crc32) Checksum algorithm to use. (crc32, adler32, md5, sha1, ipv4, udp)
+    :type  algorithm:  str, function
+    :param algorithm:  (Optional, def=crc32) Checksum algorithm to use. (crc32, crc32c, adler32, md5, sha1, ipv4, udp)
+                       Pass a function to use a custom algorithm. This function has to take and return byte-type data.
 
     :type  length:     int
-    :param length:     (Optional, def=0) NOT IMPLEMENTED. Length of checksum, specify 0 to auto-calculate
+    :param length:     (Optional, def=0) Length of checksum, auto-calculated by default. Must be specified manually when
+                       using a custom algorithm.
 
     :type  endian:     Character
     :param endian:     (Optional, def=LITTLE_ENDIAN) Endianness of the bit field (LITTLE_ENDIAN: <, BIG_ENDIAN: >)
@@ -245,9 +350,17 @@ def s_checksum(block_name, algorithm="crc32", length=0, endian=LITTLE_ENDIAN, fu
     if block_name in blocks.CURRENT.block_stack:
         raise exception.SullyRuntimeError("CAN N0T ADD A CHECKSUM FOR A BLOCK CURRENTLY IN THE STACK")
 
-    checksum = Checksum(block_name, blocks.CURRENT, algorithm, length, endian, fuzzable, name,
-                        ipv4_src_block_name=ipv4_src_block_name,
-                        ipv4_dst_block_name=ipv4_dst_block_name)
+    checksum = Checksum(
+        block_name,
+        blocks.CURRENT,
+        algorithm,
+        length,
+        endian,
+        fuzzable,
+        name,
+        ipv4_src_block_name=ipv4_src_block_name,
+        ipv4_dst_block_name=ipv4_dst_block_name,
+    )
     blocks.CURRENT.push(checksum)
 
 
@@ -260,7 +373,7 @@ def s_repeat(block_name, min_reps=0, max_reps=None, step=1, variable=None, fuzza
     :see: Aliases: s_repeater()
 
     :type  block_name: str
-    :param block_name: Name of block to apply sizer to
+    :param block_name: Name of block to repeat
     :type  min_reps:   int
     :param min_reps:   (Optional, def=0) Minimum number of block repetitions
     :type  max_reps:   int
@@ -279,8 +392,18 @@ def s_repeat(block_name, min_reps=0, max_reps=None, step=1, variable=None, fuzza
     blocks.CURRENT.push(repeat)
 
 
-def s_size(block_name, offset=0, length=4, endian=LITTLE_ENDIAN, output_format="binary", inclusive=False, signed=False,
-           math=None, fuzzable=True, name=None):
+def s_size(
+    block_name,
+    offset=0,
+    length=4,
+    endian=LITTLE_ENDIAN,
+    output_format="binary",
+    inclusive=False,
+    signed=False,
+    math=None,
+    fuzzable=True,
+    name=None,
+):
     """
     Create a sizer block bound to the block with the specified name. You *can not* create a sizer for any
     currently open blocks.
@@ -358,7 +481,7 @@ def s_binary(value, name=None):
     parsed = parsed.replace("0x", "")
     parsed = parsed.replace("\\x", "")
 
-    value = ""
+    value = b""
     while parsed:
         pair = parsed[:2]
         parsed = parsed[2:]
@@ -473,7 +596,7 @@ def s_static(value, name=None):
 def s_mirror(primitive_name, name=None):
     """
     Push a mirror of another primitive onto the current block stack.
-    
+
     :type primitive_name:   str
     :param primitive_name:  Name of target primitive
     :type name:             str
@@ -517,7 +640,7 @@ def s_from_file(value, encoding="ascii", fuzzable=True, max_len=0, name=None, fi
     :type  value:    str
     :param value:    Default string value
     :type  encoding: str
-    :param encoding: (Optonal, def="ascii") String encoding, ex: utf_16_le for Microsoft Unicode.
+    :param encoding: (DEPRECIATED, def="ascii") String encoding, ex: utf_16_le for Microsoft Unicode.
     :type  fuzzable: bool
     :param fuzzable: (Optional, def=True) Enable/disable fuzzing of this primitive
     :type  max_len:  int
@@ -528,13 +651,14 @@ def s_from_file(value, encoding="ascii", fuzzable=True, max_len=0, name=None, fi
     :param filename: (Mandatory) Specify filename where to read fuzz list
     """
 
-    s = primitives.FromFile(value, encoding, fuzzable, max_len, name, filename)
+    s = primitives.FromFile(value, fuzzable, max_len, name, filename)
     blocks.CURRENT.push(s)
 
 
 # noinspection PyTypeChecker
-def s_bit_field(value, width, endian=LITTLE_ENDIAN, output_format="binary", signed=False, full_range=False,
-                fuzzable=True, name=None):
+def s_bit_field(
+    value, width, endian=LITTLE_ENDIAN, output_format="binary", signed=False, full_range=False, fuzzable=True, name=None
+):
     """
     Push a variable length bit field onto the current block stack.
 
@@ -547,7 +671,7 @@ def s_bit_field(value, width, endian=LITTLE_ENDIAN, output_format="binary", sign
     :type  endian:         Character
     :param endian:         (Optional, def=LITTLE_ENDIAN) Endianess of the bit field (LITTLE_ENDIAN: <, BIG_ENDIAN: >)
     :type  output_format:  str
-    :param output_format format:  (Optional, def=binary) Output format, "binary" or "ascii"
+    :param output_format:  (Optional, def=binary) Output format, "binary" or "ascii"
     :type  signed:         bool
     :param signed:         (Optional, def=False) Make size signed vs. unsigned (applicable only with format="ascii")
     :type  full_range:     bool
@@ -562,8 +686,9 @@ def s_bit_field(value, width, endian=LITTLE_ENDIAN, output_format="binary", sign
     blocks.CURRENT.push(bit_field)
 
 
-def s_byte(value, endian=LITTLE_ENDIAN, output_format="binary", signed=False, full_range=False, fuzzable=True,
-           name=None):
+def s_byte(
+    value, endian=LITTLE_ENDIAN, output_format="binary", signed=False, full_range=False, fuzzable=True, name=None
+):
     """
     Push a byte onto the current block stack.
 
@@ -589,8 +714,31 @@ def s_byte(value, endian=LITTLE_ENDIAN, output_format="binary", signed=False, fu
     blocks.CURRENT.push(byte)
 
 
-def s_word(value, endian=LITTLE_ENDIAN, output_format="binary", signed=False, full_range=False, fuzzable=True,
-           name=None):
+def s_bytes(value, size=None, padding=b"\x00", fuzzable=True, max_len=None, name=None):
+    """
+    Push a bytes field onto the current block stack.
+
+    :type  value:        bytes
+    :param value:        Default binary value
+    :type  size:         int
+    :param size:         (Optional, def=None) Static size of this field, leave None for dynamic.
+    :type  padding:      chr
+    :param padding:      (Optional, def=b"\\x00") Value to use as padding to fill static field size.
+    :type  fuzzable:     bool
+    :param fuzzable:     (Optional, def=True) Enable/disable fuzzing of this primitive
+    :type  max_len:      int
+    :param max_len:      (Optional, def=None) Maximum string length
+    :type  name:         str
+    :param name:         (Optional, def=None) Specifying a name gives you direct access to a primitive
+    """
+
+    _bytes = primitives.Bytes(value, size, padding, fuzzable, max_len, name)
+    blocks.CURRENT.push(_bytes)
+
+
+def s_word(
+    value, endian=LITTLE_ENDIAN, output_format="binary", signed=False, full_range=False, fuzzable=True, name=None
+):
     """
     Push a word onto the current block stack.
 
@@ -616,8 +764,9 @@ def s_word(value, endian=LITTLE_ENDIAN, output_format="binary", signed=False, fu
     blocks.CURRENT.push(word)
 
 
-def s_dword(value, endian=LITTLE_ENDIAN, output_format="binary", signed=False, full_range=False, fuzzable=True,
-            name=None):
+def s_dword(
+    value, endian=LITTLE_ENDIAN, output_format="binary", signed=False, full_range=False, fuzzable=True, name=None
+):
     """
     Push a double word onto the current block stack.
 
@@ -643,8 +792,9 @@ def s_dword(value, endian=LITTLE_ENDIAN, output_format="binary", signed=False, f
     blocks.CURRENT.push(dword)
 
 
-def s_qword(value, endian=LITTLE_ENDIAN, output_format="binary", signed=False, full_range=False, fuzzable=True,
-            name=None):
+def s_qword(
+    value, endian=LITTLE_ENDIAN, output_format="binary", signed=False, full_range=False, fuzzable=True, name=None
+):
     """
     Push a quad word onto the current block stack.
 
