@@ -11,7 +11,11 @@ class DataTestStep(object):
     description = attr.ib()
     data = attr.ib()
     timestamp = attr.ib()
-    truncated = attr.ib(type=bool)
+    try:
+        truncated = attr.ib(type=bool)
+    except TypeError:
+        # in case attr version is too old
+        truncate = attr.ib()
 
     @property
     def text_render(self):
@@ -21,7 +25,7 @@ class DataTestStep(object):
             data=self.data,
             timestamp=self.timestamp,
             truncated=self.truncated,
-            format_type='terminal',
+            format_type="terminal",
         )
 
     @property
@@ -32,9 +36,9 @@ class DataTestStep(object):
             data=self.data,
             timestamp=self.timestamp,
             truncated=self.truncated,
-            format_type='html',
+            format_type="html",
         )
 
     @property
     def css_class(self):
-        return helpers.test_step_info[self.type]['css_class']
+        return helpers.test_step_info[self.type]["css_class"]
