@@ -13,6 +13,10 @@ class BoofuzzTargetConnectionFailedError(BoofuzzError):
     pass
 
 
+class BoofuzzOutOfAvailableSockets(BoofuzzError):
+    pass
+
+
 class BoofuzzTargetConnectionReset(BoofuzzError):
     pass
 
@@ -22,6 +26,7 @@ class BoofuzzTargetConnectionAborted(BoofuzzError):
     """
     Raised on `errno.ECONNABORTED`.
     """
+
     socket_errno = attr.ib()
     socket_errmsg = attr.ib()
 
@@ -44,3 +49,7 @@ class SizerNotUtilizedError(Exception):
 
 class MustImplementException(Exception):
     pass
+
+
+class BoofuzzSSLError(BoofuzzError):
+    pass
