@@ -1,38 +1,40 @@
 from __future__ import absolute_import, print_function
 
-import pickle
 import datetime
+import errno
 import logging
 import os
+import pickle
 import re
-import six
+import socket
 import threading
 import time
 import traceback
 import zlib
-import socket
-import errno
+from io import open
 
+import six
+from builtins import input
+from future.utils import listitems
 from tornado.httpserver import HTTPServer
 from tornado.ioloop import IOLoop
 from tornado.wsgi import WSGIContainer
 
 from boofuzz import helpers
-from . import blocks, constants
-from . import event_hook
-from . import fuzz_logger
-from . import fuzz_logger_db
-from . import fuzz_logger_text
-from . import fuzz_logger_curses
-from . import ifuzz_logger
-from . import pgraph
-from . import primitives
-from . import exception
+from . import (
+    blocks,
+    constants,
+    event_hook,
+    exception,
+    fuzz_logger,
+    fuzz_logger_curses,
+    fuzz_logger_db,
+    fuzz_logger_text,
+    pgraph,
+    primitives,
+)
 from .web.app import app
 
-from future.utils import listitems
-from io import open
-from builtins import input
 
 class Target(object):
     """Target descriptor container.
@@ -73,9 +75,9 @@ class Target(object):
 
         :return: None
         """
-        self._fuzz_data_logger.log_info('Closing target connection...')
+        self._fuzz_data_logger.log_info("Closing target connection...")
         self._target_connection.close()
-        self._fuzz_data_logger.log_info('Connection closed.')
+        self._fuzz_data_logger.log_info("Connection closed.")
 
     def open(self):
         """
@@ -83,9 +85,9 @@ class Target(object):
 
         :return: None
         """
-        self._fuzz_data_logger.log_info('Opening target connection ({0})...'.format(self._target_connection.info))
+        self._fuzz_data_logger.log_info("Opening target connection ({0})...".format(self._target_connection.info))
         self._target_connection.open()
-        self._fuzz_data_logger.log_info('Connection opened.')
+        self._fuzz_data_logger.log_info("Connection opened.")
 
     def pedrpc_connect(self):
         """
@@ -101,7 +103,7 @@ class Target(object):
 
             # connection established.
             for key, value in listitems(self.procmon_options):
-                getattr(self.procmon, 'set_{0}'.format(key))(value)
+                getattr(self.procmon, "set_{0}".format(key))(value)
 
         # If the network monitor is alive, set it's options
         if self.netmon:
@@ -203,7 +205,7 @@ class SessionInfo(object):
 
     @property
     def netmon_results(self):
-        return {1: 'netmon results not saved in current database format'}
+        return {1: "netmon results not saved in current database format"}
 
     @property
     def fuzz_node(self):
@@ -215,7 +217,7 @@ class SessionInfo(object):
 
     @property
     def total_mutant_index(self):
-        x = self._db_reader.query('''SELECT COUNT(*) FROM cases''').next()[0]
+        x = next(self._db_reader.query("SELECT COUNT(*) FROM cases"))[0]
         return x
 
     def test_case_data(self, index):
@@ -235,7 +237,7 @@ class SessionInfo(object):
 
     @property
     def state(self):
-        return 'finished'
+        return "finished"
 
 
 class WebApp(object):
@@ -246,7 +248,7 @@ class WebApp(object):
         web_port (int):         Port for monitoring fuzzing campaign via a web browser. Default 26000.
     """
 
-    def __init__(self, session_info, web_port=constants.DEFAULT_WEB_UI_PORT, web_addr='localhost'):
+    def __init__(self, session_info, web_port=constants.DEFAULT_WEB_UI_PORT, web_addr="localhost"):
         self._session_info = session_info
         self._web_interface_thread = self._build_webapp_thread(port=web_port, address=web_addr)
         pass
@@ -262,12 +264,12 @@ class WebApp(object):
     def server_init(self):
         """Called by fuzz() to initialize variables, web interface, etc.
         """
-        if not self._web_interface_thread.isAlive():
+        if not self._web_interface_thread.is_alive():
             # spawn the web interface.
             self._web_interface_thread.start()
 
 
-def open_test_run(db_filename, port=constants.DEFAULT_WEB_UI_PORT, address='localhost'):
+def open_test_run(db_filename, port=constants.DEFAULT_WEB_UI_PORT, address="localhost"):
     s = SessionInfo(db_filename=db_filename)
     w = WebApp(session_info=s, web_port=port, web_addr=address)
     w.server_init()
@@ -316,6 +318,7 @@ class Session(pgraph.Graph):
         ignore_connection_issues_when_sending_fuzz_data (bool): Ignore fuzz data transmission failures. Default True.
                                 This is usually a helpful setting to enable, as targets may drop connections once a
                                 message is clearly invalid.
+        ignore_connection_ssl_errors (bool): Log SSL related errors as "info" instead of failures. Default False.
         reuse_target_connection (bool): If True, only use one target connection instead of reconnecting each test case.
                                         Default False.
         target (Target):        Target for fuzz session. Target must be fully initialized. Default None.
@@ -328,34 +331,43 @@ class Session(pgraph.Graph):
                                 Was once used to set the log level for the logfile. Default logger.INFO.
     """
 
-    def __init__(self, session_filename=None, index_start=1, index_end=None, sleep_time=0.0,
-                 restart_interval=0,
-                 web_port=constants.DEFAULT_WEB_UI_PORT,
-                 keep_web_open=True,
-                 console_gui=False,
-                 crash_threshold_request=12,
-                 crash_threshold_element=3,
-                 restart_sleep_time=5,
-                 restart_callbacks=None,
-                 pre_send_callbacks=None,
-                 post_test_case_callbacks=None,
-                 fuzz_data_logger=None,
-                 fuzz_loggers=None,
-                 fuzz_db_keep_only_n_pass_cases=0,
-                 receive_data_after_each_request=True,
-                 check_data_received_each_request=False,
-                 receive_data_after_fuzz=False,
-                 log_level=logging.INFO, logfile=None, logfile_level=logging.DEBUG,
-                 ignore_connection_reset=False,
-                 ignore_connection_aborted=False,
-                 ignore_connection_issues_when_sending_fuzz_data=True,
-                 reuse_target_connection=False,
-                 target=None,
-                 ):
+    def __init__(
+        self,
+        session_filename=None,
+        index_start=1,
+        index_end=None,
+        sleep_time=0.0,
+        restart_interval=0,
+        web_port=constants.DEFAULT_WEB_UI_PORT,
+        keep_web_open=True,
+        console_gui=False,
+        crash_threshold_request=12,
+        crash_threshold_element=3,
+        restart_sleep_time=5,
+        restart_callbacks=None,
+        pre_send_callbacks=None,
+        post_test_case_callbacks=None,
+        fuzz_data_logger=None,
+        fuzz_loggers=None,
+        fuzz_db_keep_only_n_pass_cases=0,
+        receive_data_after_each_request=True,
+        check_data_received_each_request=False,
+        receive_data_after_fuzz=False,
+        log_level=logging.INFO,
+        logfile=None,
+        logfile_level=logging.DEBUG,
+        ignore_connection_reset=False,
+        ignore_connection_aborted=False,
+        ignore_connection_issues_when_sending_fuzz_data=True,
+        ignore_connection_ssl_errors=False,
+        reuse_target_connection=False,
+        target=None,
+    ):
         self._ignore_connection_reset = ignore_connection_reset
         self._ignore_connection_aborted = ignore_connection_aborted
         self._ignore_connection_issues_when_sending_fuzz_data = ignore_connection_issues_when_sending_fuzz_data
         self._reuse_target_connection = reuse_target_connection
+        self._ignore_connection_ssl_errors = ignore_connection_ssl_errors
         _ = log_level
         _ = logfile
         _ = logfile_level
@@ -374,22 +386,23 @@ class Session(pgraph.Graph):
         self._crash_threshold_element = crash_threshold_element
         self.restart_sleep_time = restart_sleep_time
         if fuzz_data_logger is not None:
-            raise exception.BoofuzzError('Session fuzz_data_logger is deprecated. Use fuzz_loggers instead!')
+            raise exception.BoofuzzError("Session fuzz_data_logger is deprecated. Use fuzz_loggers instead!")
         if fuzz_loggers is None:
             fuzz_loggers = []
-        if self.console_gui and os.name != 'nt':
+        if self.console_gui and os.name != "nt":
             fuzz_loggers.append(fuzz_logger_curses.FuzzLoggerCurses(web_port=self.web_port))
             self._keep_web_open = False
         if len(fuzz_loggers) == 0:
             fuzz_loggers = [fuzz_logger_text.FuzzLoggerText()]
 
         helpers.mkdir_safe(os.path.join(constants.RESULTS_DIR))
-        self._run_id = datetime.datetime.utcnow().replace(microsecond=0).isoformat().replace(':', '-')
-        self._db_filename = os.path.join(constants.RESULTS_DIR, 'run-{0}.db'.format(self._run_id))
-        self._db_logger = fuzz_logger_db.FuzzLoggerDb(db_filename=self._db_filename,
-                                                      num_log_cases=fuzz_db_keep_only_n_pass_cases)
+        self._run_id = datetime.datetime.utcnow().replace(microsecond=0).isoformat().replace(":", "-")
+        self._db_filename = os.path.join(constants.RESULTS_DIR, "run-{0}.db".format(self._run_id))
+        self._db_logger = fuzz_logger_db.FuzzLoggerDb(
+            db_filename=self._db_filename, num_log_cases=fuzz_db_keep_only_n_pass_cases
+        )
 
-        self._crash_filename = 'boofuzz-crash-bin-{0}'.format(self._run_id)
+        self._crash_filename = "boofuzz-crash-bin-{0}".format(self._run_id)
 
         self._fuzz_data_logger = fuzz_logger.FuzzLogger(fuzz_loggers=[self._db_logger] + fuzz_loggers)
         self._check_data_received_each_request = check_data_received_each_request
@@ -440,7 +453,7 @@ class Session(pgraph.Graph):
         self.add_node(self.root)
 
         if target is not None:
-            target.procmon_options['crash_filename'] = self._crash_filename
+            target.procmon_options["crash_filename"] = self._crash_filename
             try:
                 self.add_target(target=target)
             except exception.BoofuzzRpcError as e:
@@ -561,7 +574,7 @@ class Session(pgraph.Graph):
             "total_mutant_index": self.total_mutant_index,
             "netmon_results": self.netmon_results,
             "procmon_results": self.procmon_results,
-            "is_paused": self.is_paused
+            "is_paused": self.is_paused,
         }
 
         fh = open(self.session_filename, "wb+")
@@ -616,7 +629,7 @@ class Session(pgraph.Graph):
         Args:
             name (str): Name of node.
         """
-        self.fuzz_single_node_by_path(re.split('->', name))
+        self.fuzz_single_node_by_path(re.split("->", name))
 
     def fuzz_single_case(self, mutant_index):
         """Fuzz a test case by mutant_index.
@@ -664,8 +677,7 @@ class Session(pgraph.Graph):
             # exception should have already been handled but rethrown in order to escape test run
             pass
         except Exception:
-            self._fuzz_data_logger.log_error(
-                "Unexpected exception! {0}".format(traceback.format_exc()))
+            self._fuzz_data_logger.log_error("Unexpected exception! {0}".format(traceback.format_exc()))
             self.export_file()
             raise
 
@@ -694,9 +706,11 @@ class Session(pgraph.Graph):
                     break
 
                 # Check restart interval
-                if num_cases_actually_fuzzed \
-                        and self.restart_interval \
-                        and num_cases_actually_fuzzed % self.restart_interval == 0:
+                if (
+                    num_cases_actually_fuzzed
+                    and self.restart_interval
+                    and num_cases_actually_fuzzed % self.restart_interval == 0
+                ):
                     self._fuzz_data_logger.open_test_step("restart interval of %d reached" % self.restart_interval)
                     self._restart_target(self.targets[0])
 
@@ -707,8 +721,10 @@ class Session(pgraph.Graph):
                 self.targets[0].close()
 
             if self._keep_web_open and self.web_port is not None:
-                print("\nFuzzing session completed. Keeping webinterface up on localhost:{}".format(self.web_port),
-                      "\nPress ENTER to close webinterface")
+                print(
+                    "\nFuzzing session completed. Keeping webinterface up on localhost:{}".format(self.web_port),
+                    "\nPress ENTER to close webinterface",
+                )
                 input()
         except KeyboardInterrupt:
             # TODO: should wait for the end of the ongoing test case, and stop gracefully netmon and procmon
@@ -723,8 +739,7 @@ class Session(pgraph.Graph):
             # exception should have already been handled but rethrown in order to escape test run
             pass
         except Exception:
-            self._fuzz_data_logger.log_error(
-                "Unexpected exception! {0}".format(traceback.format_exc()))
+            self._fuzz_data_logger.log_error("Unexpected exception! {0}".format(traceback.format_exc()))
             self.export_file()
             raise
         finally:
@@ -812,7 +827,8 @@ class Session(pgraph.Graph):
         if target.netmon:
             captured_bytes = target.netmon.post_send()
             self._fuzz_data_logger.log_info(
-                "netmon captured %d bytes for test case #%d" % (captured_bytes, self.total_mutant_index))
+                "netmon captured %d bytes for test case #%d" % (captured_bytes, self.total_mutant_index)
+            )
             self.netmon_results[self.total_mutant_index] = captured_bytes
 
     def _check_procmon_failures(self, target):
@@ -828,8 +844,10 @@ class Session(pgraph.Graph):
                 self._fuzz_data_logger.log_pass("No crash detected.")
             else:
                 self._fuzz_data_logger.log_fail(
-                    "procmon detected crash on test case #{0}: {1}".format(self.total_mutant_index,
-                                                                           target.procmon.get_crash_synopsis()))
+                    "procmon detected crash on test case #{0}: {1}".format(
+                        self.total_mutant_index, target.procmon.get_crash_synopsis()
+                    )
+                )
                 return True
         return False
 
@@ -875,22 +893,29 @@ class Session(pgraph.Graph):
             self.procmon_results[self.total_mutant_index] = crash_synopses
             self._fuzz_data_logger.log_info(synopsis)
 
-            if self.fuzz_node.mutant is not None and \
-                    self.crashing_primitives[self.fuzz_node] >= self._crash_threshold_node:
+            if (
+                self.fuzz_node.mutant is not None
+                and self.crashing_primitives[self.fuzz_node] >= self._crash_threshold_node
+            ):
                 skipped = self.fuzz_node.num_mutations() - self.fuzz_node.mutant_index
                 self._skip_current_node_after_current_test_case = True
                 self._fuzz_data_logger.open_test_step(
-                    "Crash threshold reached for this request, exhausting {0} mutants.".format(skipped))
+                    "Crash threshold reached for this request, exhausting {0} mutants.".format(skipped)
+                )
                 self.total_mutant_index += skipped
                 self.fuzz_node.mutant_index += skipped
-            elif self.fuzz_node.mutant is not None and \
-                    self.crashing_primitives[self.fuzz_node.mutant] >= self._crash_threshold_element:
-                if not isinstance(self.fuzz_node.mutant, primitives.Group) \
-                        and not isinstance(self.fuzz_node.mutant, blocks.Repeat):
+            elif (
+                self.fuzz_node.mutant is not None
+                and self.crashing_primitives[self.fuzz_node.mutant] >= self._crash_threshold_element
+            ):
+                if not isinstance(self.fuzz_node.mutant, primitives.Group) and not isinstance(
+                    self.fuzz_node.mutant, blocks.Repeat
+                ):
                     skipped = self.fuzz_node.mutant.num_mutations() - self.fuzz_node.mutant.mutant_index
                     self._skip_current_element_after_current_test_case = True
                     self._fuzz_data_logger.open_test_step(
-                        "Crash threshold reached for this element, exhausting {0} mutants.".format(skipped))
+                        "Crash threshold reached for this element, exhausting {0} mutants.".format(skipped)
+                    )
                     self.total_mutant_index += skipped
                     self.fuzz_node.mutant_index += skipped
 
@@ -957,8 +982,9 @@ class Session(pgraph.Graph):
                     self._fuzz_data_logger.open_test_step('Pre_Send callback: "{0}"'.format(f.__name__))
                     f(target=target, fuzz_data_logger=self._fuzz_data_logger, session=self, sock=target)
             except Exception:
-                self._fuzz_data_logger.log_error(constants.ERR_CALLBACK_FUNC.format(func_name="pre_send")
-                                                 + traceback.format_exc())
+                self._fuzz_data_logger.log_error(
+                    constants.ERR_CALLBACK_FUNC.format(func_name="pre_send") + traceback.format_exc()
+                )
 
     def _restart_target(self, target):
         """
@@ -1000,13 +1026,14 @@ class Session(pgraph.Graph):
             except exception.BoofuzzRestartFailedError:
                 raise
             except Exception:
-                self._fuzz_data_logger.log_error(constants.ERR_CALLBACK_FUNC.format(func_name="restart_target")
-                                                 + traceback.format_exc())
+                self._fuzz_data_logger.log_error(
+                    constants.ERR_CALLBACK_FUNC.format(func_name="restart_target") + traceback.format_exc()
+                )
             finally:
-                self._fuzz_data_logger.open_test_step('Cleaning up connections from callbacks')
+                self._fuzz_data_logger.open_test_step("Cleaning up connections from callbacks")
                 target.close()
                 if self._reuse_target_connection:
-                    self._fuzz_data_logger.open_test_step('Reopening target connection')
+                    self._fuzz_data_logger.open_test_step("Reopening target connection")
                     target.open()
 
         # otherwise all we can do is wait a while for the target to recover on its own.
@@ -1022,7 +1049,7 @@ class Session(pgraph.Graph):
     def server_init(self):
         """Called by fuzz() to initialize variables, web interface, etc.
         """
-        if not self.web_interface_thread.isAlive():
+        if not self.web_interface_thread.is_alive():
             # spawn the web interface.
             self.web_interface_thread.start()
 
@@ -1036,7 +1063,7 @@ class Session(pgraph.Graph):
 
         # if the edge has a callback, process it. the callback has the option to render the node, modify it and return.
         if edge.callback:
-            self._fuzz_data_logger.open_test_step('Callback function')
+            self._fuzz_data_logger.open_test_step("Callback function")
             data = edge.callback(self.targets[0], self._fuzz_data_logger, session=self, node=node, edge=edge)
 
         return data
@@ -1066,12 +1093,17 @@ class Session(pgraph.Graph):
                 self._fuzz_data_logger.log_fail(constants.ERR_CONN_RESET)
         except exception.BoofuzzTargetConnectionAborted as e:
             # TODO: Switch _ignore_connection_aborted for _ignore_transmission_error, or provide retry mechanism
-            msg = constants.ERR_CONN_ABORTED.format(socket_errno=e.socket_errno,
-                                                    socket_errmsg=e.socket_errmsg)
+            msg = constants.ERR_CONN_ABORTED.format(socket_errno=e.socket_errno, socket_errmsg=e.socket_errmsg)
             if self._ignore_connection_aborted:
                 self._fuzz_data_logger.log_info(msg)
             else:
                 self._fuzz_data_logger.log_fail(msg)
+        except exception.BoofuzzSSLError as e:
+            if self._ignore_connection_ssl_errors:
+                self._fuzz_data_logger.log_info(str(e))
+            else:
+                self._fuzz_data_logger.log_fail(str(e))
+
         try:  # recv
             if self._receive_data_after_each_request:
                 self.last_recv = self.targets[0].recv()
@@ -1089,12 +1121,16 @@ class Session(pgraph.Graph):
             else:
                 self._fuzz_data_logger.log_info(constants.ERR_CONN_RESET)
         except exception.BoofuzzTargetConnectionAborted as e:
-            msg = constants.ERR_CONN_ABORTED.format(socket_errno=e.socket_errno,
-                                                    socket_errmsg=e.socket_errmsg)
+            msg = constants.ERR_CONN_ABORTED.format(socket_errno=e.socket_errno, socket_errmsg=e.socket_errmsg)
             if self._check_data_received_each_request:
                 self._fuzz_data_logger.log_fail(msg)
             else:
                 self._fuzz_data_logger.log_info(msg)
+        except exception.BoofuzzSSLError as e:
+            if self._ignore_connection_ssl_errors:
+                self._fuzz_data_logger.log_info(str(e))
+            else:
+                self._fuzz_data_logger.log_fail(str(e))
 
     def transmit_fuzz(self, sock, node, edge, callback_data):
         """Render and transmit a fuzzed node, process callbacks accordingly.
@@ -1119,12 +1155,16 @@ class Session(pgraph.Graph):
             else:
                 self._fuzz_data_logger.log_fail(constants.ERR_CONN_RESET)
         except exception.BoofuzzTargetConnectionAborted as e:
-            msg = constants.ERR_CONN_ABORTED.format(socket_errno=e.socket_errno,
-                                                    socket_errmsg=e.socket_errmsg)
+            msg = constants.ERR_CONN_ABORTED.format(socket_errno=e.socket_errno, socket_errmsg=e.socket_errmsg)
             if self._ignore_connection_issues_when_sending_fuzz_data:
                 self._fuzz_data_logger.log_info(msg)
             else:
                 self._fuzz_data_logger.log_fail(msg)
+        except exception.BoofuzzSSLError as e:
+            if self._ignore_connection_ssl_errors:
+                self._fuzz_data_logger.log_info(str(e))
+            else:
+                self._fuzz_data_logger.log_fail(str(e))
 
         try:  # recv
             if self._receive_data_after_fuzz:
@@ -1135,13 +1175,17 @@ class Session(pgraph.Graph):
             else:
                 self._fuzz_data_logger.log_info(constants.ERR_CONN_RESET)
         except exception.BoofuzzTargetConnectionAborted as e:
-            msg = constants.ERR_CONN_ABORTED.format(socket_errno=e.socket_errno,
-                                                    socket_errmsg=e.socket_errmsg)
+            msg = constants.ERR_CONN_ABORTED.format(socket_errno=e.socket_errno, socket_errmsg=e.socket_errmsg)
             if self._check_data_received_each_request:
                 self._fuzz_data_logger.log_fail(msg)
             else:
                 self._fuzz_data_logger.log_info(msg)
             pass
+        except exception.BoofuzzSSLError as e:
+            if self._ignore_connection_ssl_errors:
+                self._fuzz_data_logger.log_info(str(e))
+            else:
+                self._fuzz_data_logger.log_fail(str(e))
 
     def build_webapp_thread(self, port=constants.DEFAULT_WEB_UI_PORT):
         app.session = self
@@ -1155,7 +1199,7 @@ class Session(pgraph.Graph):
                     raise
                 port += 1
             else:
-                self._fuzz_data_logger.log_info("Web interface can be found at http://localhost:%d"%port)
+                self._fuzz_data_logger.log_info("Web interface can be found at http://localhost:%d" % port)
                 break
         flask_thread = threading.Thread(target=IOLoop.instance().start)
         flask_thread.daemon = True
@@ -1194,7 +1238,7 @@ class Session(pgraph.Graph):
             path.append(edge)
 
             message_path = "->".join([self.nodes[e.dst].name for e in path])
-            logging.debug('checking: {0}'.format(message_path))
+            logging.debug("checking: {0}".format(message_path))
 
             self.fuzz_node = self.nodes[path[-1].dst]
             self.total_mutant_index += 1
@@ -1245,7 +1289,7 @@ class Session(pgraph.Graph):
             path.append(edge)
 
             message_path = "->".join([self.nodes[e.dst].name for e in path])
-            logging.debug('fuzzing: {0}'.format(message_path))
+            logging.debug("fuzzing: {0}".format(message_path))
 
             for x in self._iterate_single_node(path):
                 yield x
@@ -1330,20 +1374,22 @@ class Session(pgraph.Graph):
 
         test_case_name = self._test_case_name_feature_check(path)
 
-        self._fuzz_data_logger.open_test_case("{0}: {1}".format(self.total_mutant_index, test_case_name),
-                                              name=test_case_name,
-                                              index=self.total_mutant_index,
-                                              num_mutations=self.total_num_mutations,
-                                              current_index=self.fuzz_node.mutant_index,
-                                              current_num_mutations=self.fuzz_node.num_mutations())
+        self._fuzz_data_logger.open_test_case(
+            "{0}: {1}".format(self.total_mutant_index, test_case_name),
+            name=test_case_name,
+            index=self.total_mutant_index,
+            num_mutations=self.total_num_mutations,
+            current_index=self.fuzz_node.mutant_index,
+            current_num_mutations=self.fuzz_node.num_mutations(),
+        )
 
         try:
             if target.procmon:
-                self._fuzz_data_logger.open_test_step('Calling procmon pre_send()')
+                self._fuzz_data_logger.open_test_step("Calling procmon pre_send()")
                 target.procmon.pre_send(self.total_mutant_index)
 
             if target.netmon:
-                self._fuzz_data_logger.open_test_step('Calling netmon pre_send()')
+                self._fuzz_data_logger.open_test_step("Calling netmon pre_send()")
                 target.netmon.pre_send(self.total_mutant_index)
 
             self._open_connection_keep_trying(target)
@@ -1376,6 +1422,7 @@ class Session(pgraph.Graph):
             else:
                 print("PASS: {0}".format(test_case_name))
             self._stop_netmon(target)
+            self._fuzz_data_logger.close_test_case()
             self.export_file()
 
     def _fuzz_current_case(self, path):
@@ -1393,26 +1440,31 @@ class Session(pgraph.Graph):
 
         test_case_name = self._test_case_name(path, self.fuzz_node.mutant)
 
-        self._fuzz_data_logger.open_test_case("{0}: {1}".format(self.total_mutant_index, test_case_name),
-                                              name=test_case_name,
-                                              index=self.total_mutant_index,
-                                              num_mutations=self.total_num_mutations,
-                                              current_index=self.fuzz_node.mutant_index,
-                                              current_num_mutations=self.fuzz_node.num_mutations())
+        self._fuzz_data_logger.open_test_case(
+            "{0}: {1}".format(self.total_mutant_index, test_case_name),
+            name=test_case_name,
+            index=self.total_mutant_index,
+            num_mutations=self.total_num_mutations,
+            current_index=self.fuzz_node.mutant_index,
+            current_num_mutations=self.fuzz_node.num_mutations(),
+        )
 
         self._fuzz_data_logger.log_info(
-            "Type: %s. Default value: %s. Case %d of %d overall." % (
+            "Type: %s. Default value: %s. Case %d of %d overall."
+            % (
                 type(self.fuzz_node.mutant).__name__,
                 repr(self.fuzz_node.mutant.original_value),
                 self.total_mutant_index,
-                self.total_num_mutations))
+                self.total_num_mutations,
+            )
+        )
 
         if target.procmon:
-            self._fuzz_data_logger.open_test_step('Calling procmon pre_send()')
+            self._fuzz_data_logger.open_test_step("Calling procmon pre_send()")
             target.procmon.pre_send(self.total_mutant_index)
 
         if target.netmon:
-            self._fuzz_data_logger.open_test_step('Calling netmon pre_send()')
+            self._fuzz_data_logger.open_test_step("Calling netmon pre_send()")
             target.netmon.pre_send(self.total_mutant_index)
 
         try:
@@ -1452,6 +1504,7 @@ class Session(pgraph.Graph):
             target (Target): Target to open.
         """
         if not self._reuse_target_connection:
+            out_of_available_sockets_count = 0
             while True:
                 try:
                     target.open()
@@ -1459,6 +1512,12 @@ class Session(pgraph.Graph):
                 except exception.BoofuzzTargetConnectionFailedError:
                     self._fuzz_data_logger.log_info(constants.WARN_CONN_FAILED_TERMINAL)
                     self._restart_target(target)
+                except exception.BoofuzzOutOfAvailableSockets:
+                    out_of_available_sockets_count += 1
+                    if out_of_available_sockets_count == 50:
+                        raise exception.BoofuzzError("There are no available sockets. Ending fuzzing.")
+                    self._fuzz_data_logger.log_info("There are no available sockets. Waiting for another 5 seconds.")
+                    time.sleep(5)
 
     def _sleep(self, seconds):
         self._fuzz_data_logger.log_info("sleeping for %f seconds" % seconds)
@@ -1473,7 +1532,7 @@ class Session(pgraph.Graph):
         if mutated_element.name:
             primitive_under_test = mutated_element.name
         else:
-            primitive_under_test = 'no-name'
+            primitive_under_test = "no-name"
         return "{0}.{1}.{2}".format(message_path, primitive_under_test, self.fuzz_node.mutant_index)
 
     def _post_send(self, target):
@@ -1485,15 +1544,22 @@ class Session(pgraph.Graph):
             except exception.BoofuzzTargetConnectionReset:
                 self._fuzz_data_logger.log_fail(constants.ERR_CONN_RESET_FAIL)
             except exception.BoofuzzTargetConnectionAborted as e:
-                self._fuzz_data_logger.log_info(constants.ERR_CONN_ABORTED.format(socket_errno=e.socket_errno,
-                                                                                  socket_errmsg=e.socket_errmsg))
+                self._fuzz_data_logger.log_info(
+                    constants.ERR_CONN_ABORTED.format(socket_errno=e.socket_errno, socket_errmsg=e.socket_errmsg)
+                )
             except exception.BoofuzzTargetConnectionFailedError:
                 self._fuzz_data_logger.log_fail(constants.ERR_CONN_FAILED)
+            except exception.BoofuzzSSLError as e:
+                if self._ignore_connection_ssl_errors:
+                    self._fuzz_data_logger.log_info(str(e))
+                else:
+                    self._fuzz_data_logger.log_fail(str(e))
             except Exception:
-                self._fuzz_data_logger.log_error(constants.ERR_CALLBACK_FUNC.format(func_name="post_send")
-                                                 + traceback.format_exc())
+                self._fuzz_data_logger.log_error(
+                    constants.ERR_CALLBACK_FUNC.format(func_name="post_send") + traceback.format_exc()
+                )
             finally:
-                self._fuzz_data_logger.open_test_step('Cleaning up connections from callbacks')
+                self._fuzz_data_logger.open_test_step("Cleaning up connections from callbacks")
 
     def _reset_fuzz_state(self):
         """
